/*
 * bg.c
 *
 * Last modified: <2008/06/25 13:48:45 +0900> By SUMIDA
 */
#include <stdio.h>
#include <stdarg.h>
#include <SDL.h>
#include "surface.h"
#include "bg.h"

BG bg[BG_MAX];

void bg_init(void) {

	int i;

	printf("bg_init ... ");

	for (i = 0; i < BG_MAX; i++) {
		BG *p = &bg[i];
		//p->priority = i * 256 / BG_MAX;
		p->ofs_x   = 8;
		p->ofs_y   = 8;
		p->font_x  = 0;
		p->font_y  = 0;
		p->frame_x = 1;
		p->frame_y = 1;
		p->frame_w = BG_MAP_WIDTH;
		p->frame_h = BG_MAP_HEIGHT;

		memset(bg[i].map, 0, sizeof(bg[i].map));
	}

	printf("done.\n");
}

void bg_rendaring(SDL_Surface *screen, int bg_no) {

	BG *p = &bg[bg_no];
	int col, row;
	int ofs_x, ofs_y;
	int cmax, rmax;
	int co, ro;

	p->ofs_x %= (BG_MAP_WIDTH  + BG_MAP_RESERVE) * BG_MAP_SIZE;
	p->ofs_y %= (BG_MAP_HEIGHT + BG_MAP_RESERVE) * BG_MAP_SIZE;
	col = p->ofs_x >> 3;
	row = p->ofs_y >> 3;
	ofs_x = (col << 3) - p->ofs_x;
	ofs_y = (row << 3) - p->ofs_y;
	cmax = (ofs_x == 0)? BG_MAP_WIDTH : BG_MAP_WIDTH  + 1;
	rmax = (ofs_y == 0)? BG_MAP_HEIGHT: BG_MAP_HEIGHT + 1;

	for (ro = 0; ro < rmax; ro++) {
		int r = (BG_MAP_HEIGHT + BG_MAP_RESERVE + row + ro) % (BG_MAP_HEIGHT + BG_MAP_RESERVE);
		int y = ofs_y + (ro << 3);

		for (co = 0; co < cmax; co++) {
			int c = (BG_MAP_WIDTH + BG_MAP_RESERVE + col + co) % (BG_MAP_WIDTH + BG_MAP_RESERVE);
			MAP_ATTR *attr = &p->map[c][r];
#if 0
			if (c < 0 || c >= BG_MAP_WIDTH + BG_MAP_RESERVE
				|| r < 0 || r >= BG_MAP_HEIGHT + BG_MAP_RESERVE) {
				fprintf(stderr, "p->ofs_x=%d, p->ofs_y=%d, c=%d, r=%d, row=%d, ro=%d, col=%d, co=%d\n", p->ofs_x, p->ofs_y, c, r, row, ro, col, co);
			}
#endif
			if (attr->map_id) {
				int x = ofs_x + (co << 3);
				surface_draw_frame8(screen, attr->surface_id, attr->map_id, x, y);
			}
		}
	}
}

void font_frame(int bg_no, int x, int y, int w, int h) {

	BG *p = &bg[bg_no];

	p->frame_x = x;
	p->frame_y = y;
	p->frame_w = w;
	p->frame_h = h;
}

void font_locate(int bg_no, int x, int y) {

	BG *p = &bg[bg_no];

	p->font_x = x;
	p->font_y = y;
}

void font_printf(int bg_no, const char *format, ...) {

	static char buf[BG_MAP_WIDTH * BG_MAP_HEIGHT];

	BG *p = &bg[bg_no];

	char *str = buf;
	char ch;

	va_list ap;

	va_start(ap, format);

	vsnprintf(buf, sizeof(buf), format, ap);

	va_end(ap);

	while (p->font_y < p->frame_h && (ch = *str++) != '\0') {

		//printf("font_x=%d, font_y=%d, frame_x=%d, frame_y=%d\n", p->font_x, p->font_y, p->frame_x, p->frame_y);

		switch (ch) {
		case ' ':
			p->font_x++;
			break;
		case '\t':
			p->font_x += 4;
			break;
		case '\n':
			p->font_x = 0;
			p->font_y++;
			break;
		default:
			p->map[p->font_x + p->frame_x][p->font_y + p->frame_y].map_id = ch;
			p->font_x++;
			break;
		}

		if (p->font_x >= p->frame_w) {
			p->font_x = 0;
			p->font_y++;
		}
	}
}
