/*
 * enemy.c
 *
 * Last modified: <2008/06/27 12:27:02 +0900> By SUMIDA
 */
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "ship.h"
#include "shot.h"
#include "enemy.h"
#include "effect.h"

extern void enemy1_init(SHIP_INFO *info);
extern void enemy2_init(SHIP_INFO *info);
extern void enemy3_init(SHIP_INFO *info);
extern void enemy4_init(SHIP_INFO *info);
extern void enemy5_init(SHIP_INFO *info);
extern void boss1_init(SHIP_INFO *info);
extern void boss2_init(SHIP_INFO *info);
extern void missile_init(SHIP_INFO *info);

extern void enemy1(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void enemy2(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void enemy3(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void enemy4(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void enemy5(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void boss1(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void boss2(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);
extern void missile(SHIP_INFO *info, SHOT_MGR *shot_mgr, SHIP_INFO *ship);

extern void enemy1_term(SHIP_INFO *info);
extern void enemy2_term(SHIP_INFO *info);
extern void enemy3_term(SHIP_INFO *info);
extern void enemy4_term(SHIP_INFO *info);
extern void enemy5_term(SHIP_INFO *info);
extern void boss1_term(SHIP_INFO *info);
extern void boss2_term(SHIP_INFO *info);
extern void missile_term(SHIP_INFO *info);

void(*func_enemy_init[])(SHIP_INFO*) = {
	enemy1_init,
	enemy2_init,
	enemy3_init,
	enemy4_init,
	enemy5_init,
	missile_init,
	boss1_init,
	boss2_init,
};

void(*func_enemy[])(SHIP_INFO*, SHOT_MGR*, SHIP_INFO*) = {
	enemy1,
	enemy2,
	enemy3,
	enemy4,
	enemy5,
	missile,
	boss1,
	boss2,
};

void(*func_enemy_term[])(SHIP_INFO*) = {
	enemy1_term,
	enemy2_term,
	enemy3_term,
	enemy4_term,
	enemy5_term,
	missile_term,
	boss1_term,
	boss2_term,
};

SHIP_INFO *enemy_new(SHIP_MGR *mgr, SHIP_ID id, Real x, Real y) {

	SHIP_INFO *info = ship_new(mgr, id);

	if (info != NULL) {

		info->stat     = SHIP_STAT_APPEAR;
		info->count    = 10;
		info->sprite.x = x;
		info->sprite.y = y;

		func_enemy_init[id - SHIP_ID_ENEMY](info);
	}

	return info;
}

void enemy_action(SHIP_MGR *mgr, SHOT_MGR *shot_mgr, SHIP_INFO *ship) {

	SHIP_INFO *prev = NULL;
	SHIP_INFO *info = mgr->active;

	while (info != NULL) {

		sprite_action(&info->sprite);

		if (!(info->flag & SHIP_FLAG_NOCLIP)) {
			SURFACE *surface = surface_get(info->sprite.surface_id);

			int x0 = RealToInt(info->sprite.x);
			int y0 = RealToInt(info->sprite.y);
			int x1 = x0 + surface->frame_width  -1;
			int y1 = y0 + surface->frame_height -1;

			if (x0 > CLIP_X1 || x1 < CLIP_X0 || y0 > CLIP_Y1 || y1 < CLIP_Y0) {
				SHIP_INFO *next = info->next;
				REPORT(format("delete(info=%p, id=%d, x0=%d, y0=%d, x1=%d, y1=%d)", info, info->id, x0, y0, x1, y1));
				ship_delete(mgr, info, prev);
				info = next;
				continue;
			}
		}

		/* JE^̃fNg */
		--info->count;

		/* Ԗ̏s */
		switch (info->stat) {
		case SHIP_STAT_APPEAR:
		case SHIP_STAT_NORMAL:
		case SHIP_STAT_MOVE:
		case SHIP_STAT_FIRE:
			if (sprite_collision_check(&info->sprite, &ship->sprite) == SDL_TRUE) {
				ship_down(ship, &info->strength, info->sprite.dx >> 2, info->sprite.dy >> 2);
			}

			if (info->stat != SHIP_STAT_EXPLOSION) {
				func_enemy[info->id - SHIP_ID_ENEMY](info, shot_mgr, ship);
			}
			break;

		case SHIP_STAT_EXPLOSION:
			if (info->count < 0) {
				info->stat = SHIP_STAT_LOST;
			}
			break;

		default:
			break;
		}

		if (info->stat == SHIP_STAT_LOST) {
			/* 폜 */
			SHIP_INFO *next = info->next;
			func_enemy_term[info->id - SHIP_ID_ENEMY](info);
			REPORT(format("delete(info=%p, id=%d)", info, info->id));
			ship_delete(mgr, info, prev);
			info = next;

		} else {
			/* eԂ  */
			info->flag &= ~SHIP_FLAG_DOWN;

			/* \ */
			sprite_put(&info->sprite);
			prev = info;
			info = info->next;
		}
	}
}
