/*
 * ship.c
 *
 * Last modified: <2008/09/07 05:16:01 +0900> By SUMIDA
 */
#include <stdlib.h>
#include <string.h>
#include <SDL.h>
#include "system.h"
#include "texture.h"
#include "surface.h"
#include "sprite.h"
#include "ship.h"
#include "effect.h"
#include "sound.h"
#include "shot.h"

#define SPRITE_OPT_SHIP  SPRITE_VISIBLE|SPRITE_FRAME|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME|SPRITE_USE_MOVE|SPRITE_USE_COLLISION
#define SPRITE_OPT_SIGHT SPRITE_VISIBLE|SPRITE_FRAME64
#define SPRITE_OPT_FORCE SPRITE_VISIBLE|SPRITE_FRAME64|SPRITE_LOOP_ANIME|SPRITE_USE_ANIME|SPRITE_USE_COLLISION

typedef struct SHIP_TYPE {
	int tex_id;
	int strength;
	int point;
	Uint32 options;
	RealRect collision;
} SHIP_TYPE;

SHIP_TYPE ship_type[] = {
	{TEX_SHIP  ,     1,    0, SPRITE_OPT_SHIP , {IntToReal(28), IntToReal(10), IntToReal(60-29) , IntToReal(60-41)}},
	{TEX_SIGHT ,     1,    0, SPRITE_OPT_SIGHT, {IntToReal(2) , IntToReal(2) , IntToReal(64-3)  , IntToReal(64-3) }},
	{TEX_FORCE ,     1,    1, SPRITE_OPT_FORCE, {IntToReal(2) , IntToReal(2) , IntToReal(64-3)  , IntToReal(64-3) }},
	{TEX_ENEMY1,     1,   10, SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY2,     2,   20, SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY3,    10,   30, SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY4,    20,   40, SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_ENEMY5,    30,   50, SPRITE_OPT_SHIP , {IntToReal(2) , IntToReal(2) , IntToReal(60-3)  , IntToReal(60-3) }},
	{TEX_MISSILE,    1,   10, SPRITE_OPT_SHIP , {IntToReal(4) , IntToReal(10), IntToReal(16-4)  , IntToReal(54-1) }},
	{TEX_BOSS1 ,   500, 1000, SPRITE_OPT_SHIP , {IntToReal(18), IntToReal(45), IntToReal(100-19), IntToReal(164)  }},
	{TEX_BOSS2 ,  1000, 5000, SPRITE_OPT_SHIP , {IntToReal(8) , IntToReal(42), IntToReal(100-9) , IntToReal(116)  }},
};

void ship_init(SHIP_MGR *mgr, SHIP_INFO *infos, size_t number) {

	size_t i;

	if (mgr != NULL) {
		memset(mgr, 0, sizeof(SHIP_MGR));
		mgr->pool = infos;
	}

	memset(infos, 0, sizeof(SHIP_INFO) * number);

	for (i = 0; i < number - 1; i++) {
		infos[i].next = &infos[i + 1];
	}
}


SHIP_INFO *ship_new(SHIP_MGR *mgr, SHIP_ID id) {

	SHIP_INFO *info = mgr->pool;

	if (info != NULL) {
		SHIP_TYPE *p = &ship_type[id];

		mgr->pool   = info->next;
		info->next  = mgr->active;
		mgr->active = info;

		info->id       = id;
		info->stat     = SHIP_STAT_NORMAL;
		info->flag     = 0;
		info->count    = 0;
		info->point    = p->point;
		info->strength = p->strength;
		info->sprite.surface_id = p->tex_id;
		info->sprite.options = p->options;
		info->sprite.frame_start    = 0;
		info->sprite.frame_end      = 1;
		info->sprite.frame_interval = 5;
		info->sprite.collision.x0   = p->collision.x0;
		info->sprite.collision.y0   = p->collision.y0;
		info->sprite.collision.x1   = p->collision.x1;
		info->sprite.collision.y1   = p->collision.y1;
	}

	return info;
}

void ship_delete(SHIP_MGR *mgr, SHIP_INFO *info, SHIP_INFO *prev) {

//	if (!(info->flag & SHIP_FLAG_NOPOOL)) {

		if (prev == NULL) {
			mgr->active = info->next;
		} else {
			prev->next = info->next;
		}

		memset(info, 0, sizeof(SHIP_INFO));
		info->next = mgr->pool;
		mgr->pool = info;
//	}
}

int ship_down(SHIP_INFO *info, int *strength, Real dx, Real dy) {

	int score = 0;

	if (info->stat != SHIP_STAT_EXPLOSION && info->stat != SHIP_STAT_LOST) {

		int pow = (*strength > SHOT_LIMIT_STRENGTH)? SHOT_LIMIT_STRENGTH: *strength;

		*strength -= pow;
		if (!(info->flag & SHIP_FLAG_NODAMAGE)) {
			info->strength -= pow;
		}

		if (info->strength > 0) {
			info->flag |= SHIP_FLAG_DOWN;
			if (!(info->flag & SHIP_FLAG_HEAVY)) {
				info->sprite.x += dx;
				info->sprite.y += dy;
			}
			sound_effect(SE_SHOT1);

		} else {
			EFFECT_INFO *effect;

			REPORT(format("bomb! (info=%p, id=%d, point=%d)", info, info->id, info->point));
			score = info->point;
			info->stat = SHIP_STAT_EXPLOSION;
			info->count = SHIP_DEF_LOST_WAIT;
			info->sprite.options &= ~SPRITE_USE_MOVE;

			switch (info->id) {
			case SHIP_ID_BOSS1:
			case SHIP_ID_BOSS2:
				effect = effect_new(EFFECT_ID_BOMB);
				sound_effect(SE_BOMB1);
				break;
			case SHIP_ID_SIGHT:
			case SHIP_ID_FORCE:
				break;
			default:
				effect = effect_new(EFFECT_ID_EXPLOSION);
				sound_effect(SE_BOMB0);
				break;
			}

			if (effect != NULL) {
				effect_set_pos(effect, &info->sprite, 0, 0);
			}
		}
	}

	return score;
}
